/******************************************************************************
 * BackgammonBoard.java - Be a backgammon board
 * $Id: BackgammonBoard.java,v 1.16 2011/06/24 01:40:09 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: BackgammonBoard.java,v $
 * Revision 1.16  2011/06/24 01:40:09  dick
 * Add isPlayerO(), just because he is not X, doesn't mean that he's O (he could be a watcher).
 *
 * Revision 1.15  2011/06/18 06:22:06  dick
 * Stop passing "dice" and "turn" all around.  In an MVC model, this board has all of that information already.
 *
 * Revision 1.14  2011/06/16 19:03:58  dick
 * parseBoard is really a BackgammonBoard function.
 * Move it there so we can do lightweight board parsing (without a Game and its baggage).
 *
 * Revision 1.13  2011/06/16 00:43:18  dick
 * Javadoc.
 *
 * Revision 1.12  2011/06/12 15:35:54  dick
 * PlayerO comes on from his bar, not X's.  This was wrong in *one* of the *two* places this attribute is managed.
 *
 * Revision 1.11  2011/06/11 19:18:34  dick
 * Stop passing jibsServer all around.  If anybody wants it, they can do JibsServer.getInstance().
 *
 * Revision 1.10  2011/06/11 04:59:04  dick
 * [sg]et[po]Die[12] becomes [sg]et[XO]Die[12].
 *
 * Revision 1.9  2011/06/09 18:01:37  dick
 * If property isDumpBoardsToStdout is set, then dump boards to stdout.
 *
 * Revision 1.8  2011/06/07 19:42:57  dick
 * onHome/Bar 1/2 becomes onHomeBar X/O
 * outBoard() takes a Player, not a JibsWriter.  -- We need to know what style the Player wants.
 * Finish crawford attributes.
 * switch2O needs to set the reverse players.
 * isRace() is a helper for greedy mode.
 *
 * Revision 1.7  2011/06/05 05:56:01  dick
 * Get rid of Player1 and 2, and MayDouble1 and 2.  Call them PlayerX and PlayerO and MayDoubleX and O.
 *
 * Revision 1.6  2011/06/03 19:26:10  dick
 * Add some debug opening boards.
 *
 * Revision 1.5  2011/06/03 17:05:48  dick
 * Define X=0 and O=1 so we can get playerScore[X] and playerScore[O].
 *
 * Revision 1.4  2011/06/03 04:14:07  dick
 * Attributes player[12]Got becomes player[12]Score.
 *
 * Revision 1.3  2011/06/01 20:26:18  dick
 * Deadwood delete.  Privatize privates.  Javadoc.
 *
 * Revision 1.2  2011/05/30 05:29:21  dick
 * retab.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.backgammon;

import java.util.StringTokenizer;

import net.sourceforge.jibs.server.JibsServer;
import net.sourceforge.jibs.server.Player;
import net.sourceforge.jibs.util.JibsSet;

/**
 * This is the encapsulation of a backgammon board as needed for <b>Jibs</b>.<br>
 * Therefore it also models a match between two players of several games.
 * Each game uses a <i>real</i> Backgammon board to represent such a
 * particular game.<br>
 * One of the players is playing the X-pieces (represemted by negative
 * numbers) whereas the other is playing the O-pieces (represented by
 * positive numbers).<br>
 * <br>
 * This class uses an array of 26 ints to represemt the board internally.
 * 
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.16 $ <br> $Date: 2011/06/24 01:40:09 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/backgammon/BackgammonBoard.java">cvs BackgammonBoard.java</a>
 * @see <a href="http://www.fibs.com/fibs_interface.html#board_state" target="_top">FIBS Client Protocol Detailed Specification</a>
 */
public class BackgammonBoard {
	//private final static boolean DEBUG = true;

	/** Index into the X player = 0 */
	public static final int X = 0;
	/** Index into the O player = 1 */
	public static final int O = 1;

	/** Player X's turn = -1 */
	public static final int turnX = -1;
	/** Player O's turn = 1*/
	public static final int turnO =  1;
	
	//	private static Logger logger = Logger.getLogger(BackgammonBoard.class);
	char[] value = new char[] {13,10};
	public String newline = new String(value);
	private Player playerX;
	private Player playerO;
	private Die dieX1;
	private Die dieX2;
	private Die dieO1;
	private Die dieO2;
	private int bar;
	private int[] board;
	private int canMove;
	private int color;
	private int cubeNumber;
	private int didCrawford;
	private int direction;
	private int forcedMove;
	private int home;
	private int matchlength;
	private int mayDoubleX;
	private int mayDoubleO;
	private int onBarX;
	private int onBarO;
	private int onHomeX;
	private int onHomeO;
	int[] playerScore = new int[2];
	String[] playerName = new String[2];
	private int redoubles;
	private int turn;
	private int wasDoubled;
	private JibsMatch jibsMatchVersion;
	private Player joinPlayer1;
	private Player joinPlayer2;
	private boolean crawFordGame = false;
	private	boolean	postCrawfordGame = false;
	private boolean isEnded = false;

	/** Default bean constructor */
	public BackgammonBoard() {
		init();
	}
	
	/** Construct a clone of the passed board
	 * @param b2 The board to make a copy of.
	 */
	public BackgammonBoard(final BackgammonBoard b2) {
		this.playerX = b2.playerX;
		this.playerO = b2.playerO;
		this.playerName[X] = b2.playerName[X];
		this.playerName[O] = b2.playerName[O];
		this.matchlength = b2.matchlength;
		this.jibsMatchVersion = b2.jibsMatchVersion;
		this.init();

		this.dieX1.setValue(b2.dieX1.getValue());
		this.dieX2.setValue(b2.dieX2.getValue());
		this.dieO1.setValue(b2.dieO1.getValue());
		this.dieO2.setValue(b2.dieO2.getValue());
		this.bar = b2.bar;
		this.canMove = b2.canMove;
		this.color = b2.color;
		this.cubeNumber = b2.cubeNumber;
		this.didCrawford = b2.didCrawford;
		this.direction = b2.direction;
		this.forcedMove = b2.forcedMove;
		this.home = b2.home;
		this.mayDoubleX = b2.mayDoubleX;
		this.mayDoubleO = b2.mayDoubleO;
		this.onBarX = b2.onBarX;
		this.onBarO = b2.onBarO;
		this.onHomeX = b2.onHomeX;
		this.onHomeO = b2.onHomeO;
//		this.playerName[X] = b2.playerName[X];
//		this.playerName[O] = b2.playerName[O];
		this.playerScore[O] = b2.playerScore[O];
		this.playerScore[X] = b2.playerScore[X];
		this.redoubles = b2.redoubles;
		this.turn = b2.turn;
		this.wasDoubled = b2.wasDoubled;
		this.joinPlayer1 = b2.joinPlayer1;
		this.joinPlayer2 = b2.joinPlayer2;
		this.crawFordGame = b2.crawFordGame;
		this.postCrawfordGame = b2.postCrawfordGame;

		for (int i = 0; i < 26; i++) {
			this.board[i] = b2.board[i];
		}

		this.isEnded = b2.isEnded;
	}

	/** Construct a board with these values.
	 * @param playerXName
	 * @param playerOName
	 * @param matchlength
	 * @param matchVersion
	 */
	public BackgammonBoard(JibsServer jibsServer, Player playerX, Player playerO,
			int matchlength, JibsMatch matchVersion) {

		this.playerX = playerX;
		this.playerO = playerO;
		this.playerName[X] = playerX.getName();
		this.playerName[O] = playerO.getName();
		this.matchlength = matchlength;
		this.jibsMatchVersion = matchVersion;


		this.init();
	}

	/** Create a copy of this board from the opponent's point of view.
	 * @return The mirror of this board
	 */
	public BackgammonBoard switch2O() {
		BackgammonBoard opBoard = new BackgammonBoard(this);
		opBoard.setPlayerXName(getPlayerOName());
		opBoard.setPlayerOName(getPlayerXName());
		opBoard.setPlayerX(getPlayerO());
		opBoard.setPlayerO(getPlayerX());
		opBoard.setDirection(getDirection() == 1 ? -1 : 1);
		opBoard.setColor(getColor() == 1 ? 0 : 1);
		opBoard.getXDie1().setValue(getODie1().getValue());
		opBoard.getXDie2().setValue(getODie2().getValue());
		opBoard.getODie1().setValue(getXDie1().getValue());
		opBoard.getODie2().setValue(getXDie2().getValue());
		opBoard.setOnBarX(getOnBarO());
		opBoard.setOnBarO(getOnBarX());
		opBoard.setOnHomeX(getOnHomeO());
		opBoard.setOnHomeO(getOnHomeX());
		opBoard.setBar(getHome());
		opBoard.setHome(getBar());
		opBoard.setPlayerXScore(getPlayerOScore());
		opBoard.setPlayerOScore(getPlayerXScore());
		opBoard.setMayDoubleX(getMayDoubleO());
		opBoard.setMayDoubleO(getMayDoubleX());

		return opBoard;
	}

	/**
	 */
	private void init() {
		dieX1 = new Die();
		dieX2 = new Die();
		dieO1 = new Die();
		dieO2 = new Die();


		playerScore[O] = 0;
		playerScore[X] = 0;
		board = new int[26];
		setInitialPositions();

		this.isEnded = false;
	}

	public void setInitialPositions() {

		// standard backgammon board
		for (int i = 0; i < board.length; i++) {
			board[i] = 0;
		}

		// Standard opening
		board[24] = 2;
		board[13] = 5;
		board[8] = 3;
		board[6] = 5;
		board[1] = -2;
		board[12] = -5;
		board[17] = -3;
		board[19] = -5;

/*
		board[1]=-2; board[3] = 2; board[4] = 2; board[6] = 3; board[8] = 1;
		board[12] = -4; board[13] = 5; board[17] = -4; board[19] = -5;
		board[24] = 2; //board[0]=2; // Bar von 'X' //board[25]=2; // Bar von 'O'
*/
/*		// Debug for a gammon
		board[1] = 4;
		board[4] = -5;
		board[5] = -5;
		board[6] = -5;
*/
/*		// Debug for a gammon the other way
		board[24] = -4;
		board[23] = 5;
		board[22] = 5;
		board[21] = 5;
*/
		cubeNumber = 1;
		forcedMove = 0;
		didCrawford = 0;
		redoubles = 0;
		mayDoubleX = 1;
		mayDoubleO = 1;
		wasDoubled = 0;
		color = -1;
		direction = 1;
		home = 25;
		bar = 0;

		// board[0]=2; // Bar 'X'
		int totalPlayer1 = board[0];
		int totalPlayer2 = board[25];

		for (int i = 1; i <= 24; i++) {
			if (board[i] > 0) {
				totalPlayer2 += Math.abs(board[i]);
			}

			if (board[i] < 0) {
				totalPlayer1 += Math.abs(board[i]);
			}
		}

		onHomeX = 15 - totalPlayer1;
		onHomeO = 15 - totalPlayer2;

		onBarX = board[0];
		onBarO = board[25];
		
	}

	/**
	 *
	 * @return
	 */
	public int getBar() {
		return bar;
	}

	/**
	 *
	 * @return
	 */
	public int[] getBoard() {
		return board;
	}

	/**
	 *
	 * @return
	 */
	public int getCanMove() {
		return canMove;
	}

	/**
	 *
	 * @return
	 */
	public int getColor() {
		return color;
	}

	/**
	 *
	 * @return
	 */
	public int getCubeNumber() {
		return cubeNumber;
	}

	/**
	 *
	 * @return
	 */
	public int getDidCrawford() {
		return didCrawford;
	}

	/**
	 *
	 * @return
	 */
	public int getDirection() {
		return direction;
	}

	/**
	 *
	 */
	public int getForcedMove() {
		return forcedMove;
	}

	/**
	 *
	 * @return
	 */
	public int getHome() {
		return home;
	}

	/**
	 *
	 * @return
	 */
	public int getMatchlength() {
		return matchlength;
	}

	/** Is PlayerX allowed to double
	 * @return 1 = yes, 0 = no
	 */
	public int getMayDoubleX() {
		return mayDoubleX;
	}

	/** Is PlayerX allowed to double
	 * @return 1 = yes, 0 = no
	 */
	public int getMayDoubleO() {
		return mayDoubleO;
	}

	public boolean getMayDouble(Player player) {
		if (player == playerX)
			return(mayDoubleX == 1);
		if (player == playerO)
			return(mayDoubleO == 1);
		throw new RuntimeException("Can't match player: " + player.getName());
		
	}

	/** Can PlayerX double?
	 * @param mayDouble 1 = true
	 */
	public void setMayDoubleX(int mayDouble) {
		this.mayDoubleX = mayDouble;
	}

	/** Can PlayerO double?
	 * @param mayDouble 1 = true
	 */
	public void setMayDoubleO(int mayDouble) {
		this.mayDoubleO = mayDouble;
	}

	
	/**
	 *
	 * @return
	 */
	public int getOnBarX() {
		return onBarX;
	}

	/**
	 *
	 * @return
	 */
	public int getOnBarO() {
		return onBarO;
	}

	/**
	 *
	 * @return
	 */
	public int getOnHomeX() {
		return onHomeX;
	}

	/**
	 *
	 * @return
	 */
	public int getOnHomeO() {
		return onHomeO;
	}

	public int getOnHome(int XO) {
		if (XO == X)
			return(onHomeX);
		return(onHomeO);
	}

	/** Get a boardstyle=3 board of the current state of the game
	 * @param game The game to turn into a string
	 * @return A Fibs type 3 board.
	 */
	public String getOutputBoard(JibsGame game) {
		StringBuffer strBoard = new StringBuffer();

		strBoard.append("board:");
		strBoard.append("You:");
		strBoard.append(playerName[O]);
		strBoard.append(":");
		strBoard.append(matchlength);
		strBoard.append(":");
		strBoard.append(playerScore[X]);
		strBoard.append(":");
		strBoard.append(playerScore[O]);

		for (int i = 0; i < 26; i++) {
			strBoard.append(":");
			strBoard.append(board[i]);
		}

		strBoard.append(":");
		strBoard.append(turn);
		strBoard.append(":");
		strBoard.append(getXDie1().getValue());
		strBoard.append(":");
		strBoard.append(getXDie2().getValue());
		strBoard.append(":");
		strBoard.append(getODie1().getValue());
		strBoard.append(":");
		strBoard.append(getODie2().getValue());
		strBoard.append(":");
		strBoard.append(cubeNumber);
		strBoard.append(":");
		strBoard.append(mayDoubleX);
		strBoard.append(":");
		strBoard.append(mayDoubleO);
		strBoard.append(":");
		strBoard.append(wasDoubled);
		strBoard.append(":");
		strBoard.append(color);
		strBoard.append(":");
		strBoard.append(direction);
		strBoard.append(":");
		strBoard.append(home);
		strBoard.append(":");
		strBoard.append(bar);
		strBoard.append(":");
		strBoard.append(onHomeX);
		strBoard.append(":");
		strBoard.append(onHomeO);
		strBoard.append(":");
		strBoard.append(onBarX);
		strBoard.append(":");
		strBoard.append(onBarO);
		strBoard.append(":");
		strBoard.append(canMove);
		strBoard.append(":");
		strBoard.append(forcedMove);
		strBoard.append(":");
		strBoard.append(didCrawford);
		strBoard.append(":");
		strBoard.append(redoubles);

		return strBoard.toString();
	}

	
	public boolean parseBoard(String s) {
		StringTokenizer tokenizer = new StringTokenizer(s, ":");
		int position = 0;
		boolean bError = false;
		int help;

		while (tokenizer.hasMoreTokens()) {
			String x = tokenizer.nextToken();

			switch (position) {
			case 0:

				if (!x.equals("board")) {
					bError = true;
				}
				break;

			case 1:
				this.setPlayerX(playerX);
				if (playerX != null) 
					this.setPlayerXName(playerX.getName());
				else
					setPlayerXName(x);
				if (!x.equals("You")) {
					bError = true;
				}
				break;

			case 2:

				Player p = JibsServer.getInstance().getPlayer(x);
				setPlayerO(p);
				this.setPlayerOName(x);
				break;

			case 3:
				help = Integer.valueOf(x);
				this.setMatchlength(help);
				break;

			case 4:
				help = Integer.valueOf(x);
				this.setPlayerXScore(help);
				break;

			case 5:
				help = Integer.valueOf(x);
				this.setPlayerOScore(help);
				break;

			case 6:
			case 7:
			case 8:
			case 9:
			case 10:
			case 11:
			case 12:
			case 13:
			case 14:
			case 15:
			case 16:
			case 17:
			case 18:
			case 19:
			case 20:
			case 21:
			case 22:
			case 23:
			case 24:
			case 25:
			case 26:
			case 27:
			case 28:
			case 29:
			case 30:
			case 31:
				help = Integer.valueOf(x);
				this.getBoard()[position - 6] = help;

				break;

			case 32: // turn

				int turn = Integer.valueOf(x);
				this.setTurn(turn);

				break;

			case 33: // player dice
				help = Integer.valueOf(x);
				this.getXDie1().setValue(help);

				break;

			case 34: // player dice
				help = Integer.valueOf(x);
				this.getXDie2().setValue(help);

				break;

			case 35: // opp dice
				help = Integer.valueOf(x);
				this.getODie1().setValue(help);

				break;

			case 36: // opp dice
				help = Integer.valueOf(x);
				this.getODie2().setValue(help);

				break;

			case 37: // cube
				help = Integer.valueOf(x);
				this.setCubeNumber(help);

				break;

			case 38: // maydouble1
				help = Integer.valueOf(x);
				this.setMayDoubleX(help);

				break;

			case 39: // maydouble2
				help = Integer.valueOf(x);
				this.setMayDoubleO(help);

				break;

			case 40: // wasdoubled
				help = Integer.valueOf(x);
				this.setWasDoubled(help);

				break;

			case 41: // color
				help = Integer.valueOf(x);
				this.setColor(help);

				break;

			case 42: // direction
				help = Integer.valueOf(x);
				this.setDirection(help);

				break;

			case 43: // home
				help = Integer.valueOf(x);
				this.setHome(help);

				break;

			case 44: // bar
				help = Integer.valueOf(x);
				this.setBar(help);

				break;

			case 45: // player onHome
				help = Integer.valueOf(x);
				this.setOnHomeX(help);

				break;

			case 46: // opp onHome
				help = Integer.valueOf(x);
				this.setOnHomeO(help);

				break;

			case 47: // player onBar
				help = new Integer(x).intValue();
				this.setOnBarX(help);

				break;

			case 48: // opp onBar
				help = Integer.valueOf(x);
				this.setOnBarO(help);

				break;

			case 49: // canMove
				help = Integer.valueOf(x);
				this.setCanMove(help);

				break;

			case 50: // forced move
				help = Integer.valueOf(x);
				this.setForcedMove(help);

				break;

			case 51: // did crawford
				help = Integer.valueOf(x);
				this.setDidCrawford(help);

				break;

			case 52: // redoubles
				help = new Integer(x).intValue();
				this.setRedoubles(help);

				break;
			}

			position++;
		}

		if (tokenizer.hasMoreTokens()) {
			bError = true;
		}
		return(bError);
	}
	/**
	 *
	 * @return
	 */
	public int getRedoubles() {
		return redoubles;
	}

	/** Who's turn is it?
	 *
	 * @return -1 = X, 1 = O
	 */
	public int getTurn() {
		return turn;
	}

	/**
	 *
	 * @return
	 */
	public int getWasDoubled() {
		return wasDoubled;
	}

	/** Output a board in the style that the player wants.
	 * @param player The player to write to
	 * @param name The name to write as the player
	 */
	public void outBoard(Player player, String name) {
		JibsSet jibsSet = player.getJibsSet();
		String strBoardStyle = (String) jibsSet.get("boardstyle");
		int boardstyle = new Integer(strBoardStyle).intValue();

		switch (boardstyle) {
		case 1:default:
		case 2:
			outBoard_2(player);
			break;

		case 3:

			String x = outBoard_3(player, name);
			player.println(x);
			break;
		}
	}

	/** Output a type 2 board, whatever the heck that is.
	 * (I only know of 1 and 3)
	 * @param out1
	 * @param turn
	 * @param dice11
	 * @param dice12
	 * @param dice21
	 * @param dice22
	 */
	private void outBoard_2(Player player) {
		int lines;
		int i;
		StringBuffer bf = new StringBuffer();

		bf.append(newline);
		bf.append("+-1--2--3--4--5--6-------7--8--9-10-11-12-+");
		bf.append(" o:");
		bf.append(playerName[O]);
		bf.append(" - score: ");
		bf.append(playerScore[O]);

		for (lines = 1; lines <= 5; lines++) {
			bf.append(newline);
			bf.append("|");

			for (i = 1; i <= 6; i++) {
				int which = board[i];
				String symbol = "   ";

				if (which < 0) {
					symbol = " x ";
				} else {
					if (which > 0) {
						symbol = " o ";
					}
				}

				if (Math.abs(which) >= lines) {
					bf.append(symbol);
				} else {
					symbol = "   ";
					bf.append(symbol);
				}
			}

			bf.append("|   |");

			for (i = 7; i <= 12; i++) {
				int which = board[i];
				String symbol = "   ";

				if (which < 0) {
					symbol = " x ";
				} else {
					if (which > 0) {
						symbol = " o ";
					}
				}

				if (Math.abs(which) >= lines) {
					bf.append(symbol);
				} else {
					symbol = "   ";
					bf.append(symbol);
				}
			}

			bf.append("|");
		}

		bf.append(newline);
		bf.append("|                  |BAR|                  | v ");

		if (getMatchVersion().getVersion() == JibsMatch.nPointMatch) {
			bf.append(matchlength);
			bf.append("-point match");
		} else {
			bf.append("unlimited match");
		}

		for (lines = 5; lines >= 1; lines--) {
			bf.append("\r\n|");

			for (i = 24; i >= 19; i--) {
				int which = board[i];
				String symbol = "   ";

				if (which < 0) {
					symbol = " x ";
				} else {
					if (which > 0) {
						symbol = " o ";
					}
				}

				if (Math.abs(which) >= lines) {
					bf.append(symbol);
				} else {
					symbol = "   ";
					bf.append(symbol);
				}
			}

			bf.append("|   |");

			for (i = 18; i >= 13; i--) {
				int which = board[i];
				String symbol = "   ";

				if (which < 0) {
					symbol = " x ";
				} else {
					if (which > 0) {
						symbol = " o ";
					}
				}

				if (Math.abs(which) >= lines) {
					bf.append(symbol);
				} else {
					symbol = "   ";
					bf.append(symbol);
				}
			}

			bf.append("|");
		}

		bf.append(newline);
		bf.append("+24-23-22-21-20-19------18-17-16-15-14-13-+");
		bf.append(" x:");
		bf.append(playerName[X]);
		bf.append(" - score: ");
		bf.append(playerScore[X]);
		bf.append(newline);
		bf.append("BAR: O:");
		bf.append(onBarO);
		bf.append(" X:");
		bf.append(onBarX);
		bf.append("   OFF: O:");
		bf.append(onHomeO);
		bf.append(" X:");
		bf.append(onHomeX);
		bf.append("   Cube: ");
		bf.append(cubeNumber);

		if (dieX1.getValue() > 0) {
			bf.append("   You rolled " + dieX1.getValue() + " and " + dieX2.getValue() + ".");
		}

		player.println(bf.toString());
	}

	/** Get a string that represents a type 3 board
	 * @param player The player whose side of the board we are
	 * @param name The name to use for the player (could be "You").
	 * @return A FIBS board string
	 */
	private String outBoard_3(Player player, String name) {
		StringBuffer strBoard = new StringBuffer();

		strBoard.append("board:");
		strBoard.append(name);
		strBoard.append(":");
		strBoard.append(getPlayerOName());
		strBoard.append(":");

		if (getMatchVersion().getVersion() == JibsMatch.nPointMatch) {
			strBoard.append(matchlength);
		} else {
			strBoard.append("9999");
		}

		strBoard.append(":");
		strBoard.append(playerScore[X]);
		strBoard.append(":");
		strBoard.append(playerScore[O]);

		for (int i = 0; i < 26; i++) {
			strBoard.append(":");
			strBoard.append(board[i]);
		}

		strBoard.append(":");
		strBoard.append(turn);
		strBoard.append(":");
		strBoard.append(dieX1.getValue());
		strBoard.append(":");
		strBoard.append(dieX2.getValue());
		strBoard.append(":");
		strBoard.append(dieO1.getValue());
		strBoard.append(":");
		strBoard.append(dieO2.getValue());
		strBoard.append(":");
		strBoard.append(cubeNumber);
		strBoard.append(":");
		strBoard.append(mayDoubleX);
		strBoard.append(":");
		strBoard.append(mayDoubleO);
		strBoard.append(":");
		strBoard.append(wasDoubled);
		strBoard.append(":");
		strBoard.append(color);
		strBoard.append(":");
		strBoard.append(direction);
		strBoard.append(":");
		strBoard.append(home);
		strBoard.append(":");
		strBoard.append(bar);
		strBoard.append(":");
		strBoard.append(onHomeX);
		strBoard.append(":");
		strBoard.append(onHomeO);
		strBoard.append(":");
		strBoard.append(onBarX);
		strBoard.append(":");
		strBoard.append(onBarO);
		strBoard.append(":");
		strBoard.append(canMove);
		strBoard.append(":");
		strBoard.append(forcedMove);
		strBoard.append(":");
		strBoard.append(didCrawford);
		strBoard.append(":");
		strBoard.append(redoubles);

		String s = strBoard.toString();
		if (JibsServer.getInstance().isDumpBoardsToStdout()) {
			System.out.println(s);
			// System.out.println("b[0/25]=" + board[0] + "/" + board[25] + " bar=" + this.onBarX + "/" + this.onBarO);
		}
		return(s);
	}

	/**
	 *
	 * @param mv
	 * @return
	 */
	public BackgammonBoard placeMoveO(Move mv) {
		BackgammonBoard helpBoard = new BackgammonBoard(this);
		int source = mv.source;
		int destination = mv.destination;

		if (source == 25) { // moving from bar
			helpBoard.board[25]--;				// FIXME: maintaining an attribute in 2 distinct places is poor.
			helpBoard.onBarO--;
		} else {
			helpBoard.board[source]--;
		}

		if (destination <= 0) {
			// bearing off
			helpBoard.onHomeO++;
		} else {
			helpBoard.board[destination]++;

			if (helpBoard.board[destination] == 0) {
				helpBoard.board[destination]++;
				helpBoard.board[0]++;
				helpBoard.onBarX++;
			}

			if (destination <= 0) {
				helpBoard.onHomeO++;
			}
		}

		return helpBoard;
	}

	/**
	 * Perform a move on this board.
	 *
	 * @param mv
	 *            a half-move
	 *
	 * @return A modified backgammon board where the move has been placed on.
	 */
	public BackgammonBoard placeMoveX(Move mv) {
		BackgammonBoard helpBoard = new BackgammonBoard(this);
		int source = mv.source;
		int destination = mv.destination;

		if (source == bar) { // moving from bar	// XXX: This seems wrong.  Using bar as a const?? maybe?
			helpBoard.board[0]--;
			helpBoard.onBarX--;
		} else if (source >= 1 && source <= 24) {
			helpBoard.board[source]++;
		}

		if (destination <= 0) {
			// bearing off
			helpBoard.onHomeX++;
		} else {
			if ((destination >= 1) && (destination <= 24)) {
				helpBoard.board[destination]--;

				if (helpBoard.board[destination] == 0) {
					helpBoard.board[destination]--;
					helpBoard.board[25]++;
					helpBoard.onBarO++;
				}
			}

			if (destination >= 25) {
				helpBoard.onHomeX++;
			}
		}

		return helpBoard;
	}

	/**
	 *
	 * @param bar
	 */
	public void setBar(int bar) {
		this.bar = bar;
	}

	/**
	 *
	 * @param board
	 */
	public void setBoard(int[] board) {
		this.board = board;
	}

	/**
	 *
	 * @param canMove
	 */
	public void setCanMove(int canMove) {
		this.canMove = canMove;
	}

	/**
	 *
	 * @param color
	 */
	public void setColor(int color) {
		this.color = color;
	}

	/**
	 *
	 * @param cubeNumber
	 */
	public void setCubeNumber(int cubeNumber) {
		this.cubeNumber = cubeNumber;
	}

	/**
	 *
	 * @param didCrawford
	 */
	public void setDidCrawford(int didCrawford) {
		this.didCrawford = didCrawford;
	}

	/**
	 *
	 * @param direction
	 */
	public void setDirection(int direction) {
		this.direction = direction;
	}

	/**
	 *
	 * @param forcedMove
	 */
	public void setForcedMove(int forcedMove) {
		this.forcedMove = forcedMove;
	}

	/**
	 *
	 * @param home
	 */
	public void setHome(int home) {
		this.home = home;
	}

	/**
	 *
	 * @param matchlength
	 */
	public void setMatchlength(int matchlength) {
		this.matchlength = matchlength;
	}

	/**
	 *
	 * @param onBar1
	 */
	public void setOnBarX(int onBarX) {
		this.onBarX = onBarX;
	}

	/**
	 *
	 * @param onBar2
	 */
	public void setOnBarO(int onBarO) {
		this.onBarO = onBarO;
	}

	/**
	 *
	 * @param onHome1
	 */
	public void setOnHomeX(int onHomeX) {
		this.onHomeX = onHomeX;
	}

	/**
	 *
	 * @param onHome2
	 */
	public void setOnHomeO(int onHome2) {
		this.onHomeO = onHome2;
	}

	/**
	 *
	 * @param player1Score
	 */
	public void setPlayerScore(int XO, int playerScore) {
		this.playerScore[XO] = playerScore;
	}

	/** Add these points to player1's score.
	 * @param XO These points go to either player X or player O
	 * @param points The number of points he won this game.
	 */
	public void addPlayerScore(int XO, int points) {
		this.playerScore[XO] += points;
	}

	/** Get one of the players scores
	 * @param XO Do we want player X or player O's score
	 * @return Player X's or Player O's score
	 */
	public int getPlayerScore(int XO) {
		return(this.playerScore[XO]);
	}

	/**
	 *
	 * @param score
	 */
	public void setPlayerXScore(int score) {
		playerScore[X] = score;
	}

	/**
	 *
	 * @param score
	 */
	public void setPlayerOScore(int score) {
		playerScore[O] = score;
	}

	/**
	 *
	 * @return
	 */
	public String getPlayerXName() {
		return playerName[X];
	}

	/**
	 *
	 * @param player1Name
	 */
	public void setPlayerXName(String player1Name) {
		this.playerName[X] = player1Name;
	}


	/**
	 *
	 * @return
	 */
	public String getPlayerOName() {
		return playerName[O];
	}


	/**
	 *
	 * @param player2Score
	 */
//	public void setPlayer2Score(int player2Score) {
//		this.player2Score = player2Score;
//	}

	/** Add these points to player2's score.
	 * @param points The number of points he won this game.
	 */
//	public void addPlayer2Score(int points) {
//		this.player2Score += points;
//	}

	/**
	 *
	 * @param playerOName
	 */
	public void setPlayerOName(String playerOName) {
		this.playerName[O] = playerOName;
	}

	/**
	 *
	 * @param redoubles
	 */
	public void setRedoubles(int redoubles) {
		this.redoubles = redoubles;
	}

	/**
	 *
	 * @param turn
	 */
	public void setTurn(int turn) {
		this.turn = turn;
	}

	/**
	 *
	 * @param wasDoubled
	 */
	public void setWasDoubled(int wasDoubled) {
		this.wasDoubled = wasDoubled;
	}

	/**
	 *
	 * @param matchVersion
	 */
	public void setMatchVersion(JibsMatch matchVersion) {
		this.jibsMatchVersion = matchVersion;
	}

	/** Set PlayerX's first die
	 * @param d1 The Die to set
	 */
	public void setXDie1(Die d1) {
		dieX1.setValue(d1.getValue());
	}

	/** Get PlayerX's first die
	 * @return PlayerX's first die
	 */
	public Die getXDie1() {
		return dieX1;
	}

	/** Set PlayerX's second die
	 * @param d2 PlayerX's second die
	 */
	public void setXDie2(Die d2) {
		dieX2.setValue(d2.getValue());
	}

	/** Get PlayerX's second die
	 * @return PlayerX's second die
	 */
	public Die getXDie2() {
		return dieX2;
	}
	
	/** Set PlayerO's first die
	 * @param die The die to set
	 */
	public void setODie1(Die d1) {
		dieO1.setValue(d1.getValue());
	}

	/** Get PlayerO's first die
	 * @return PlayerO's first die
	 */
	public Die getODie1() {
		return dieO1;
	}

	/** Set PlayerO's second die
	 * @param die The die to set
	 */
	public void setODie2(Die die) {
		dieO2.setValue(die.getValue());
	}

	/** Get PlayerO's second die
	 * @return PlayerO's second die
	 */
	public Die getODie2() {
		return dieO2;
	}

	/** Get a handle to the first player, PlayerX
	 * @return The Player playing as PlayerX
	 */
	public Player getPlayerX() {
		return playerX;
	}

	/**
	 *
	 * @param playerX
	 */
	public void setPlayerX(Player playerX) {
		this.playerX = playerX;
	}

	/**
	 *
	 * @param playerO
	 */
	public void setPlayerO(Player playerO) {
		this.playerO = playerO;
	}

	/**
	 *
	 * @return
	 */
	public Player getPlayerO() {
		return playerO;
	}

	/**
	 *
	 * @return
	 */
	public JibsMatch getMatchVersion() {
		return jibsMatchVersion;
	}

	/**
	 *
	 * @param player
	 * @return
	 */
	public Player getOpponent(Player player) {
		if (player.getName().equals(playerX.getName())) {
			return playerO;
		}

		if (player.getName().equals(playerO.getName())) {
			return playerX;
		}

		return null;
	}

	/**
	 *
	 * @return
	 */
	public int getPlayerXScore() {
		return playerScore[X];
	}

	/**
	 *
	 * @return
	 */
	public int getPlayerOScore() {
		return playerScore[O];
	}

	/** Is this Player playing as PlayerX
	 * @param player The Player to test
	 * @return true if he is PlayerX, false if he is PlayerO
	 */
	public boolean isPlayerX(Player player) {
		return(player == this.playerX);
	}

	/** Is this Player playing as PlayerX
	 * @param player The Player to test
	 * @return true if he is PlayerX, false if he is PlayerO
	 */
	public boolean isPlayerO(Player player) {
		return(player == this.playerO);
	}

	/** Is this game over?
	 * @return true if the game is done.
	 */
	public boolean isEnded() {
		return isEnded;
	}

	public void setEnded(boolean isEnded) {
		this.isEnded = isEnded;
	}

	/**
	 *
	 * @param player
	 */
	public void setJoinPlayer(Player player) {
		if (joinPlayer1 == null) {
			joinPlayer1 = player;
		} else {
			joinPlayer2 = player;
		}
	}

	/**
	 *
	 * @return
	 */
	public Object getJoinPlayer1() {
		return joinPlayer1;
	}

	/**
	 *
	 * @return
	 */
	public Object getJoinPlayer2() {
		return joinPlayer2;
	}

	/** Is this a post-crawford game? (anyone can double)
	 * @return true if this is post-crawford.
	 */
	public boolean isPostCrawfordGame() {
		return(this.postCrawfordGame);
	}

	/** The game after a crawford game is post-crawford. <br>
	 * This shuts down the checking for crawford.
	 * @param b true if we have finished a crawford game.
	 */
	public void setPostCrawFordGame(boolean b) {
		this.postCrawfordGame = b;
	}

	/** Is this a crawford game?
	 * @return true if this is a crawford game.
	 */
	public boolean isCrawFordGame() {
		return(this.crawFordGame);
	}

	/** Set the state of crawfordness.
	 * @param b true = this is a crawford game.
	 */
	public void setCrawFordGame(boolean b) {
		crawFordGame = b;
	}

	/**
	 *
	 * @param object
	 */
	public void setJoinPlayer1(Player object) {
		joinPlayer1 = object;
	}

	/**
	 *
	 * @param object
	 */
	public void setJoinPlayer2(Player object) {
		joinPlayer2 = object;
	}

	/** Is this game a race?
	 * Determine whether all of one Player's checkers are past the opponent's checkers
	 * @return true if this game has turned into a race.
	 */
	public boolean isRace() {
		int i;
		int k = 0;
		if (onBarO > 0 || onBarX > 0)
			return(false);
		boolean oppositeSeen = false;
		for (i=1; i<=24; i++) {
			if (this.board[i] == 0)
				continue;
			if (k == 0) {
				k = this.board[i];
				continue;
			}
			if (!oppositeSeen) {
				if (k < 0 && this.board[i] > 0) {
					oppositeSeen = true;
					continue;
				}
				if (k > 0 && this.board[i] < 0) {
					oppositeSeen = true;
					continue;
				}
			} else {
				if (k < 0 && this.board[i] < 0)
					return(false);
				if (k > 0 && this.board[i] > 0)
					return(false);
			}
		}
		return(true);
	}

	/** Zero out any dice rolled */
	public void clearDice() {
		dieX1.setValue(0);
		dieX2.setValue(0);
		dieO1.setValue(0);
		dieO2.setValue(0);
	}

	///////////////////////////////////////////////////////////////////////////
	// positions of the fields in a board line
	private final static int YOU = 1;
	private final static int SOMEPLAYER = 2;
	private final static int MATCHLENGTH = 3;
	private final static int CURRENTSCORES = 4;
	private final static int BOARD = 6;
	private final static int TURN = 32;
	private final static int DICE = 33;
	private final static int CUBE = 37;
	private final static int MAYDOUBLE = 38;
	private final static int WASDOUBLED = 40;
	private final static int XO = 41;
	private final static int DIRECTION = 42;
	private final static int ONHOME = 45;
	private final static int ONBAR = 47;
	private final static int CANMOVE = 49;

	/** Parse the board state from Fibs.
	 * Opening board looks like this:<br>
	 * board:You:someplayer:3:0:0:0:-2:0:0:0:0:5:0:3:0:0:0:-5:5:0:0:0:-3:0:-5:0:0:0:0:2:0:1:6:2:0:0:1:1:1:0:1:-1:0:25:0:0:0:0:2:0:0:0
	 * @param s The board
	 * @see <a href="http://www.fibs.com/fibs_interface.html#board_state" target="_top">FIBS Client Protocol Detailed Specification</a>
	 * <pre>
	 * 0                                   1                    2                     3                   4                     5
	 * 0     1   2          3 4 5 6 7  8 9 0 1 2 3 4 5 6 7 8  9 0 1 2 3  4 5  6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2  3 4  5 6 7 8 9 0 1 2 
	 * board:You:someplayer:3:0:0:0:-2:0:0:0:0:5:0:3:0:0:0:-5:5:0:0:0:-3:0:-5:0:0:0:0:2:0:1:6:2:0:0:1:1:1:0:1:-1:0:25:0:0:0:0:2:0:0:0
	 * </pre>
	 */
	public void debugParseBoard(String s) {
		int i;
		String[] t = s.split(":");
		this.direction = Integer.parseInt(t[XO]);
		direction = Integer.parseInt(t[DIRECTION]);
		playerName[0] = t[YOU];
		playerName[1] = t[SOMEPLAYER];
		this.matchlength = Integer.parseInt(t[MATCHLENGTH]);
		this.playerScore[0] = Integer.parseInt(t[CURRENTSCORES+0]);
		this.playerScore[1] = Integer.parseInt(t[CURRENTSCORES+1]);
		for (i=1; i<=24; i++) {
			this.board[i] = Integer.parseInt(t[BOARD+i]) * -direction;
		}
		this.turn = Integer.parseInt(t[TURN]);
		this.dieX1.setValue(Integer.parseInt(t[DICE+0]));
		this.dieX2.setValue(Integer.parseInt(t[DICE+1]));
		this.dieO1.setValue(Integer.parseInt(t[DICE+2]));
		this.dieO2.setValue(Integer.parseInt(t[DICE+3]));
		this.cubeNumber = Integer.parseInt(t[CUBE]);
		this.mayDoubleX = Integer.parseInt(t[MAYDOUBLE+0]);
		this.mayDoubleO = Integer.parseInt(t[MAYDOUBLE+1]);
		this.wasDoubled = Integer.parseInt(t[WASDOUBLED]);
		this.onHomeX = Integer.parseInt(t[ONHOME+0]);
		this.onHomeO = Integer.parseInt(t[ONHOME+1]);
		this.onBarX = Integer.parseInt(t[ONBAR+0]);
		this.onBarO = Integer.parseInt(t[ONBAR+1]);
		this.canMove = Integer.parseInt(t[CANMOVE]);
	}
	
}
