/******************************************************************************
 * Board_Command.java - Send a Board to a client.
 * $Id: Board_Command.java,v 1.6 2011/06/18 06:31:14 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: Board_Command.java,v $
 * Revision 1.6  2011/06/18 06:31:14  dick
 * Stop passing "dice" and "turn" all around.  In an MVC model, this board has all of that information already.
 * Give a friendly message instead of a board if you are not watching or playing.
 *
 * Revision 1.5  2011/06/11 04:59:04  dick
 * [sg]et[po]Die[12] becomes [sg]et[XO]Die[12].
 *
 * Revision 1.4  2011/06/07 20:15:23  dick
 * outBoard() takes a Player, not a JibsWriter.
 *
 * Revision 1.3  2011/06/05 05:53:47  dick
 * Watchers can make a board command, too.
 *
 * Revision 1.2  2011/05/31 13:02:43  dick
 * Bucko headers.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.command;

import net.sourceforge.jibs.backgammon.BackgammonBoard;
import net.sourceforge.jibs.backgammon.JibsGame;
import net.sourceforge.jibs.server.JibsServer;
import net.sourceforge.jibs.server.Player;

/** Send a Board to a client.
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.6 $ <br> $Date: 2011/06/18 06:31:14 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/command/Board_Command.java">cvs Board_Command.java</a>
 */
public class Board_Command implements JibsCommand {
	public boolean execute(JibsServer jibsServer, Player player,
			String strArgs, String[] args) {
		JibsGame game = player.getGame();
		String pname = "You";
		Player playerXO = player;
		if (game == null) {
			playerXO = player.getWatchingPlayer();
			if (playerXO == null) {
				// m_you_not_playing=** You're not playing.
				String msg = jibsServer.getJibsMessages().convert("m_you_not_playing");
				player.println(msg);
				return(true);
			}
			game = playerXO.getGame();
			pname = playerXO.getName();
			
		}
		if (game != null) {
			BackgammonBoard board = game.getBackgammonBoard();

			if (board.isPlayerX(playerXO)) {
				board.outBoard(player, pname);
			} else {
				BackgammonBoard opBoard = board.switch2O();
				opBoard.outBoard(player, pname);
			}
		}

		return true;
	}
}
