/******************************************************************************
 * Exit_Command.java - Client is leaving.
 * $Id: Exit_Command.java,v 1.11 2011/06/24 01:41:11 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: Exit_Command.java,v $
 * Revision 1.11  2011/06/24 01:41:11  dick
 * Clean up exit notifications.
 *
 * Revision 1.10  2011/06/18 06:32:09  dick
 * When a player exits, differentiate between logout and drop.
 *
 * Revision 1.9  2011/06/11 19:18:35  dick
 * Stop passing jibsServer all around.  If anybody wants it, they can do JibsServer.getInstance().
 *
 * Revision 1.8  2011/06/09 18:19:17  dick
 * Use Player.println() instead of the JibsWriter.
 *
 * Revision 1.7  2011/06/06 03:18:25  dick
 * JibsMessages belongs in n.s.jibs.server, not n.s.jibs.gui
 *
 * Revision 1.6  2011/06/06 02:47:07  dick
 * getAllClients() comes from JibsServer now.
 *
 * Revision 1.5  2011/06/05 18:26:33  dick
 * log is a method of JibsServer.  Always write screen logs to the log file.
 * So, no need to pass write-to-logs=true from everywhere.
 *
 * Revision 1.4  2011/06/02 19:29:43  dick
 * When a Player exits the server, he is no longer a watcher.
 *
 * Revision 1.3  2011/05/31 13:02:42  dick
 * Bucko headers.
 *
 * Revision 1.2  2011/05/30 04:22:05  dick
 * Read the logout file from the resources by default.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.command;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import net.sourceforge.jibs.backgammon.JibsGame;
import net.sourceforge.jibs.server.ClientWorker;
import net.sourceforge.jibs.server.JibsMessages;
import net.sourceforge.jibs.server.JibsServer;
import net.sourceforge.jibs.server.Player;
import net.sourceforge.jibs.util.ClipConstants;

/** A client has left the building.
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.11 $ <br> $Date: 2011/06/24 01:41:11 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/command/Exit_Command.java">cvs Exit_Command.java</a>
 */
public class Exit_Command implements JibsCommand {
	
	@Override
	public boolean execute(JibsServer jibsServer, Player player,
			String strArgs, String[] args) {
		JibsMessages jibsMessages = jibsServer.getJibsMessages();
		player.stopWatching();
		try {
			if (player != null) {
				displayLogoff(jibsServer, player);
			}

			JibsGame game = player.getGame();

			if (game != null) {
				if (!game.getBackgammonBoard().isEnded()) {
					game.save();
				}
			}

			ClientWorker cw = player.getClientWorker();

			if (cw != null) {
				player.getClientWorker().getSocket().close();
				player.getClientWorker().stopWatchThread();
				player.getClientWorker().disConnectPlayer(player);
				cw.getSocket().close();
			}

			// m_player_logs_out=%0 logs out.
			Object[] obj = new Object[] { player.getName() };
			String msg; 
			if (args.length == 2)
				msg = jibsMessages.convert(args[1], obj);
			else
				msg = jibsMessages.convert("m_player_logs_out", obj);
			jibsServer.log(msg);

			for (Player curPlayer : jibsServer.getAllClients().values()) {
				//msg = player.getName() + " " + msg;
				if (curPlayer.canCLIP())
					curPlayer.println(ClipConstants.CLIP_LOGOUT + " " + player.getName() + " " + msg);
				else
					curPlayer.println(msg);
			}
		} catch (FileNotFoundException e) {
			jibsServer.logException(e);
		} catch (IOException e) {
			jibsServer.logException(e);
		}

		return false;
	}
	
	public void displayLogoff(JibsServer jibsServer, Player player) {
		String sName = jibsServer.getResource("logout");
		File file = new File(sName);
		BufferedReader inp = null;
		;

		try {
			if (file.canRead()) {
				inp = new BufferedReader(new FileReader(file));
			} else {
				InputStream is = ClassLoader.getSystemResourceAsStream("conf/logout.txt");
				if (is == null)		// busted jar file can't find the logout resource.
					return;
				inp = new BufferedReader(new InputStreamReader(is));
			}


			String theLine = inp.readLine();

			while (theLine != null) {
				if (theLine.equals("")) {
					theLine = " ";
				}

				player.println(theLine);
				theLine = inp.readLine();
			}

			inp.close();
		} catch (FileNotFoundException e) {
			jibsServer.logException(e);
		} catch (IOException e) {
			jibsServer.logException(e);
		} finally {
			try {
				if (inp != null) {
					inp.close();
				}
			} catch (IOException e) {
				jibsServer.logException(e);
			}
		}
	}

	
}
