/******************************************************************************
 * JibsGui - The main Jibs Gui window
 * $Id: JibsGui.java,v 1.3 2011/05/31 14:15:54 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: JibsGui.java,v $
 * Revision 1.3  2011/05/31 14:15:54  dick
 * Fix cvs link.
 *
 * Revision 1.2  2011/05/29 05:17:14  dick
 * Read the form data from the resources.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.gui;

import javax.swing.AbstractButton;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.table.TableColumn;

import net.sourceforge.jibs.server.JibsServer;

import com.jeta.forms.components.panel.FormPanel;
import com.jeta.forms.gui.common.FormException;
import com.jeta.forms.gui.form.FormAccessor;

/** The primaty Jibs Gui window
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.3 $ <br> $Date: 2011/05/31 14:15:54 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/gui/JibsGui.java">cvs JibsGui.java</a>
 */
public class JibsGui extends FormPanel {
	private static final long serialVersionUID = 3800005237698172527L;
	private static final int LOGINCOLUMN = 6;
	private static final int LOGOUTCOLUMN = 7;
	private JTextArea txtArea;
	private JTextField txtPlayer;
	private JLabel lblTime;
	private JibsUserTableModel jibsUserTableModel;
	private JButton btnInfo;
	private JButton btnExit;
	private JButton btnReload;
	private JButton btnStart;
	private JButton btnStop;
	private JButton btnClear;

	public JibsGui(JibsServer jibsServer) throws FormException {
		super(ClassLoader.getSystemResourceAsStream("frm/jibs.jfrm"));

		// switch the Textarea with my implementation JibsTextArea
		FormAccessor console = getFormAccessor("playerconsole");
		txtArea = (JTextArea) console.getTextComponent("txtArea");

		String maxLines = jibsServer.getResource("maxConsoleLines");
		JibsDocument doc = new JibsDocument(Integer.valueOf(maxLines));
		txtArea.setDocument(doc);
		jibsServer.setDoc(doc);

		// get components from the statusbar
		FormAccessor status = getFormAccessor("status");
		txtPlayer = status.getTextField("player");
		lblTime = status.getLabel("time");

		// switch the JTable into proper format
		JTable table = getTable("jibsTable");
		jibsUserTableModel = new JibsUserTableModel();

		TableSorter sorter = new TableSorter(jibsUserTableModel);
		table.setModel(sorter);

		int[] widhts = { 210, 100, 100, 230, 60, 60, 160, 160, 160 };

		for (int i = 0; i < jibsUserTableModel.getColumnCount(); i++) {
			TableColumn column = table.getColumnModel().getColumn(i);

			if (i == JibsGui.LOGINCOLUMN) {
				column.setCellRenderer(new DateCellRenderer());
			}

			if (i == JibsGui.LOGOUTCOLUMN) {
				column.setCellRenderer(new DateCellRenderer());
			}

			column.setPreferredWidth(widhts[i]);
		}

		sorter.setTableHeader(table.getTableHeader());
		table.setModel(sorter);

		// ease adding actions later on
		btnInfo = (JButton) getButton("btnInfo");
		btnExit = (JButton) getButton("btnExit");
		btnReload = (JButton) getButton("btnReload");
		btnStart = (JButton) getButton("btnStart");
		btnStop = (JButton) getButton("btnStop");
	}

	public JTextArea getTextArea() {
		return txtArea;
	}

	public JLabel getStatusDate() {
		return lblTime;
	}

	public JTextField getStatusPlayer() {
		return txtPlayer;
	}

	public JibsUserTableModel getUserTableModel() {
		return jibsUserTableModel;
	}

	public AbstractButton getBtnReload() {
		return btnReload;
	}

	public AbstractButton getBtnInfo() {
		return btnInfo;
	}

	public AbstractButton getBtnExit() {
		return btnExit;
	}

	public AbstractButton getBtnStart() {
		return btnStart;
	}

	public AbstractButton getBtnStop() {
		return btnStop;
	}

	public JButton getBtnClear() {
		return btnClear;
	}

	public void setBtnClear(JButton btnClear) {
		this.btnClear = btnClear;
	}
}
