/******************************************************************************
 * JibsShutdown - Thread to shutdown jIBS
 * $Id: JibsShutdown.java,v 1.3 2011/06/11 19:18:35 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: JibsShutdown.java,v $
 * Revision 1.3  2011/06/11 19:18:35  dick
 * Stop passing jibsServer all around.  If anybody wants it, they can do JibsServer.getInstance().
 *
 * Revision 1.2  2011/05/31 15:25:49  dick
 * Bucko headers.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.util;

import net.sourceforge.jibs.gui.ExitAction;
import net.sourceforge.jibs.server.JibsServer;

import java.awt.event.ActionEvent;

import java.util.Date;

/** Thread to shutdown jIBS
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.3 $ <br> $Date: 2011/06/11 19:18:35 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/util/JibsShutdown.java">cvs JibsShutdown.java</a>
 */
public class JibsShutdown extends Thread {
	private JibsServer jibsServer;
	private int delay;
	private boolean bRestart;
	private Date shutdownDate;

	public JibsShutdown(boolean bRestart, int msecs) {
		this.delay = msecs;
		this.bRestart = bRestart;
		shutdownDate = new Date(new Date().getTime() + msecs);
		this.jibsServer = JibsServer.getInstance();
	}

	public void run() {
		try {
			Thread.sleep(delay);
			jibsServer.stopServerMenuItemActionPerformed(null);

			ExitAction exitAction = new ExitAction(jibsServer, "Exit", null,
					"", null);

			if (bRestart) {
				String restart = jibsServer.getResource("Restart");
				ActionEvent evt = new ActionEvent(Integer.valueOf(restart), 0,
				"exit");
				exitAction.actionPerformed(evt);
			} else {
				// don't restart
				ActionEvent evt = new ActionEvent(Integer.valueOf(0), 0, "exit");
				exitAction.actionPerformed(evt);
			}
		} catch (InterruptedException e) {
			// don't log the exception on purpose
			// jibsServer.logException(e);
		}
	}

	public Date getShutdownDate() {
		return shutdownDate;
	}
}
